=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2009-02-26 14:10:58 +0000
+++ loggerhead/controllers/annotate_ui.py	2009-02-26 17:50:46 +0000
@@ -26,7 +26,7 @@
 from paste.httpexceptions import HTTPBadRequest, HTTPServerError
 
 from loggerhead.controllers import TemplatedBranchView
-from loggerhead.highlight import PygmentHighlighter
+from loggerhead.highlight import PygmentsHighlighter
 from loggerhead import util
 
 
@@ -40,15 +40,18 @@
         parity = 0
 
         file_revid = self._history.get_inventory(revid)[file_id].revision
-	file_name  = os.path.basename(self._history.get_path(revid, file_id))
         tree = self._history._branch.repository.revision_tree(file_revid)
+	
+	file_name = os.path.basename(self._history.get_path(revid, file_id))
+	hl_lines = None
 
         try:
 	    file_lines = tree.get_file_lines(file_id)
 
             bzrlib.textfile.check_text_lines(file_lines)
 
-	    ph = PygmentHighlighter(file_name, '\n'.join(file_lines[:128]))
+	    hl_lines = PygmentsHighlighter.highlight(file_name, 
+						     ''.join(file_lines))
         except bzrlib.errors.BinaryFile:
                 # bail out; this isn't displayable text
                 yield util.Container(parity=0, lineno=1, status='same',
@@ -74,7 +77,7 @@
 
                 yield util.Container(
                     parity=parity, lineno=lineno, status=status,
-                    change=change, text=ph.highlight(text))
+                    change=change, text=hl_lines[lineno - 1])
                 lineno += 1
 
         self.log.debug('annotate: %r secs' % (time.time() - z))

=== modified file 'loggerhead/highlight.py'
--- loggerhead/highlight.py	2009-02-26 14:10:58 +0000
+++ loggerhead/highlight.py	2009-02-26 17:50:46 +0000
@@ -26,29 +26,39 @@
 DEFAULT_PYGMENT_STYLE = 'colorful'
 
 
-class PygmentHighlighter:
-    def __init__(self, path, text, style=DEFAULT_PYGMENT_STYLE):
-	self.formatter = PygmentHtmlFormatter(style=style)
-
-	try:
-	    self.lexer = guess_lexer_for_filename(path, text, stripall=False)
+class PygmentsHighlighter:
+    @staticmethod
+    def highlight(path, text, style=DEFAULT_PYGMENT_STYLE):
+	""" 
+	Returns a list of highlighted (i.e. HTML formatted) strings and it
+	replaces initial spaces with nonbreaking spaces to maintain
+	indentation.
+	"""
+
+	formatter = PygmentsHtmlFormatter(style=style)
+	
+	encoding = 'utf-8'
+	try:
+	    text = text.decode(encoding)
+	except UnicodeDecodeError:
+	    text = text.decode(encoding)
+	    encoding = 'iso-8859-15'
+
+	try:
+	    lexer = guess_lexer_for_filename(path, text, encoding=encoding)
 	except (ClassNotFound, ValueError):
 	    try: 
-		self.lexer = guess_lexer(text, stripall=False)
+		lexer = guess_lexer(text, encoding=encoding)
 	    except (ClassNotFound, ValueError):
-		self.lexer = TextLexer(stripall=False)
-
-    def highlight(self, text):
-	hl_text  = highlight(text.expandtabs(), self.lexer, self.formatter)
-	hl_split = hl_text.find('<')
-
-	hl_text  = hl_text[:hl_split].replace(' ', util.NONBREAKING_SPACE) + \
-		   hl_text[hl_split:]
-
-	return hl_text
-
-
-class PygmentHtmlFormatter(HtmlFormatter):
+		lexer = TextLexer(encoding=encoding)
+
+	hl_lines = highlight(text, lexer, formatter).split('\n')
+	hl_lines = [ util.fix_whitespace(line) for line in hl_lines ]
+
+	return hl_lines
+
+
+class PygmentsHtmlFormatter(HtmlFormatter):
     def wrap(self, source, outfile):
 	return self._wrap_code(source)
 

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-10-24 02:26:05 +0000
+++ loggerhead/util.py	2009-02-26 17:50:46 +0000
@@ -241,6 +241,28 @@
     return s.expandtabs().replace(' ', NONBREAKING_SPACE)
 
 
+def fix_whitespace(s):
+    """
+    Replace spaces outside of HTML tags with nonbreaking spaces 
+    """
+
+    ns = ''
+    outside_tag = True	# Whether or not we are outside of a HTML tag
+
+    for c in s.expandtabs():
+	if outside_tag:
+	    if c == '<':
+		outside_tag = False
+	    elif c == ' ':
+		c = NONBREAKING_SPACE
+	else: # Inside tag
+	    if c == '>':
+		outside_tag = True
+	ns += c
+
+    return ns
+
+
 def fake_permissions(kind, executable):
     # fake up unix-style permissions given only a "kind" and executable bit
     if kind == 'directory':

